#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	Monte Carlo integration.
	We calculate the x-section for gamma* p -> qqbar + X 
      using xg(x) = 3(1-x)**5 
      and plot the distribution in pt
      Now we use the small t approximation in the calculation of the matrix 
      element and also pt

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      


*/

double pdf(double x)
{	
	double result;
      
      result = 0.;
      if(x>1e-10) {
	result = 3*pow((1-x),5)/x;
	}
      return(result);
}
double M2(double sh, double th, double uh, double q2)
{	
	static const double pi = 3.14159265358979323848;
	double aem=1./137., as = 0.3, eq, bracket, z ;
	double result;
      
      eq = 2.*(1./9. + 4./9.); 
      z = q2/(sh+q2);
// here insert the small t approximated expression for the ME
	return(result); 
}
double sigma_full(double y, double q2, double sh, double th)
{	
	static const double pi = 3.14159265358979323848;
	static const double gev2nb=0.3893857E+6;
	double result,uh;
      result = 0;
      uh = -q2 -sh -th;
// calculate dsigma/dt
	result = 1./16./pi/(sh+q2)/sh*M2(sh,th,uh,q2);
      result = result * gev2nb;
	return(result); 
}

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 10000000;
 	double y,q2;
      double s=320.*320.;
      double xg,xgmin=1e-2,xgmax=1,gluon;
      double sh, shmin=10.;
      double th, tmin=1.,tmax, pt2;
      double weightt, weightxg;
      double sum0, sum00, f, ff, sigma2, error;
#define LVEC 4
	double rvec[LVEC];
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("x","xg",100, -5, 0.);
      TH1F *histo2 = new TH1F("pt approximate","pt approximate",100, 0, 150.);
      
      sum0 = sum00 = 0;
      
      y = 0.3 ;
      q2 = 10.;
      
	for ( n1 = 0; n1 < npoints; n1++ ) {
      	ranlxd(rvec,LVEC);
// this is using importance sampling
//  here generate xg according to a 1/xg distribution and calcualte the weights
            sh = -q2 + y*xg*s;
// we treat th positiv, for the that generation            
            tmax = abs(-q2 -sh) ;
            if(tmax > tmin ) {
            f = 0; 
//  here generate t according to a 1/th distribution and calcualte uh
            	if ( xg < 1) {
            		f=sigma_full(y,q2,sh,th)*weightt; }
            		ff = f;
            		sum0 = sum0 + ff;
            		sum00 = sum00 + ff*ff; 
// weighting with 1/x0:
//	plot dxg(x)/dlogx *Jacobian, Jacobian dlogx/dx = 1/x
				histo1->Fill(log10(xg),1/xg);
				histo2->Fill(sqrt(pt2),ff);
           		}
	}
//                                  

      sum0 = sum0/npoints;
      sum00 = sum00/npoints;
      sigma2 = sum00 - sum0*sum0 ;
      error = sqrt(sigma2/npoints) ;
      cout<< " integral for gamma+p-> q+qbar + X xsection is [nb]: " << sum0<< " +/- " << error<< endl;

     double binwidth, fact;
// normalise histogram: sigma / npoints / binwidth 
      binwidth = histo1 -> GetBinWidth(5);
      fact = 1./binwidth/npoints;
//  histogram normalisation
      histo1 -> Add((TH1*)histo1,(TH1*)histo1,fact,0.);
      binwidth = histo2 -> GetBinWidth(5);
      fact = 1./binwidth/npoints;
//  histogram normalisation
      histo2 -> Add((TH1*)histo2,(TH1*)histo2,fact,0.);
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(2,1);
//enter the first part of the canvas (upper left)
      c -> cd(1);
      gPad->SetLogy();
      histo1 -> Draw();
      c -> cd(2);
      gPad->SetLogy();
      histo2 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example10.pdf");
      gMyRootApp->SetReturnFromRun(true);


//    write histogramm out to file
      TFile file("output-example10.root","RECREATE");
      histo1->Write();
      histo2->Write();
      file.Close();
	return(0);
}
