#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	Monte Carlo integration.
	We calculate the DIS x-section and plot the distribution in y and Q2

      We use the LHAPDF library to access the different PDF parameterisations.
      We check also for the LO* parameterisations.

      We use LHAPDF 

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      


*/

double f2(double x, double q2 )
{	

	double result, eq[7], Q, pdf,charge;
      
	int i,j;

      eq[0] = 0 ;
      eq[1] = eq[3] = eq[5] = -1./3.;
      eq[2] = eq[4] = eq[6] = 2./3.;
      
      result = 0.;
      Q = sqrt(q2);
      for ( i = -6; i <= 6 ; i++ ) {
      if(x>1e-10) {
// here insert F2
         }
      }
	return(result); 
}
double sigma(double y, double q2, double s)
{	
	static const double pi = 3.14159265358979323848;
	double gev2nb=0.3893857E+6, aem=1./137.;
	double x,result;
      x=q2/y/s;
      result = 0;
      if ( x < 1 ) {
// here insert the cross section 
      result = result * gev2nb;}
	return(result); 
}

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
  	const int SUBSET = 0;
      const string NAME = "MRST2004nlo";
//  	const string NAME = "MRST2007lomod";		// LO * pdf //
//  	const string NAME = "MRSTMCal";			// LO** pdf //
  	LHAPDF::initPDFSet(NAME, LHAPDF::LHGRID, SUBSET);

  	const double mz=91.2;
	int n1, npoints = 100000;
 	double y, ymin=1e-3,ymax=0.9, weighty;
 	double q2, q2min=5,q2max=100, weightq2;
      double s=320*320;
      double sum0, sum00, f, ff, sigma2, error;
#define LVEC 2
	double rvec[LVEC];
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("y","y",100, 0, 1.);
      TH1F *histo2 = new TH1F("q2","q2",100, 0, 150.);

  	cout << "alphas(mz) = " << LHAPDF::alphasPDF(mz) << endl;
  	cout << "qcdlam4    = " << LHAPDF::getLam4(SUBSET) << endl;
  	cout << "qcdlam5    = " << LHAPDF::getLam5(SUBSET) << endl;
  	cout << "orderPDF   = " << LHAPDF::getOrderPDF() << endl;
  	cout << "xmin       = " << LHAPDF::getXmin(SUBSET) << endl;
  	cout << "xmax       = " << LHAPDF::getXmax(SUBSET) << endl;
  	cout << "q2min      = " << LHAPDF::getQ2min(SUBSET) << endl;
  	cout << "q2max      = " << LHAPDF::getQ2max(SUBSET) << endl;
  	cout << "orderalfas = " << LHAPDF::getOrderAlphaS() << endl;
  	cout << "num flav   = " << LHAPDF::getNf() << endl;
  	cout << "name       = " << NAME << endl;
  	cout << "number     = " << LHAPDF::numberPDF() << endl;
  	cout << endl;

//  	const int NUMBER = numberPDF();
    	initPDF(0);

      
      sum0 = sum00 = 0;
            
	for ( n1 = 0; n1 < npoints; n1++ ) {
      	ranlxd(rvec,LVEC);
// this is using importance sampling
// here calcualte the integral with impeortance sampling over y,q2

            sum0 = sum0 + ff;
            sum00 = sum00 + ff*ff; 
		histo1->Fill(y,ff);
		histo2->Fill(q2,ff);
           }
//                                  

      sum0 = sum0/npoints;
      sum00 = sum00/npoints;
      sigma2 = sum00 - sum0*sum0 ;
      error = sqrt(sigma2/npoints) ;
      printf (" integral for DIS xsection is:  %10.8f +/- %10.8f \n",sum0,error );
     double binwidth, fact;
// normalise histogram: sigma / npoints / binwidth 
      binwidth = histo1 -> GetBinWidth(5);
      fact = sum0/binwidth/npoints;
//  histogram normalisation
      histo1 -> Add((TH1*)histo1,(TH1*)histo1,fact,0.);
      binwidth = histo2 -> GetBinWidth(5);
      fact = sum0/binwidth/npoints;
//  histogram normalisation
      histo2 -> Add((TH1*)histo2,(TH1*)histo2,fact,0.);
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(2,1);
//enter the first part of the canvas (upper left)
      c -> cd(1);
      gPad->SetLogy();
      histo1 -> Draw();
      c -> cd(2);
      gPad->SetLogy();
      histo2 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example7.pdf");
      gMyRootApp->SetReturnFromRun(true);




//    write histogramm out to file
      TFile file("output-example7.root","RECREATE");
      histo1->Write();
      histo2->Write();
      file.Close();
	return(0);
}
